package de.avat.tools.tags;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Collections;
import java.util.List;

/**
 * <p/>
 * Date: May 7, 2007<br>
 * Time: 1:59:39 PM<br>
 */
public class SingleTaggable implements Taggable {
  private final TagChangeSupport tcs;
  private Tag selectedTag;

  public SingleTaggable( @NotNull Object source ) {
    tcs = new TagChangeSupport( source );
  }

  public void setSelectedTag( @Nullable Tag tag ) {
    addTag( tag );
  }

  public boolean addTag( @Nullable Tag tag ) {
    //noinspection ObjectEquality
    if ( tag == selectedTag ) {
      return false;
    }

    if ( selectedTag != null ) {
      removeTag( selectedTag );
    }

    if ( tag != null ) {
      this.selectedTag = tag;
      tcs.notifyTagAdded( tag, 0 );
    }
    return true;
  }

  public boolean removeTag( @NotNull Tag tag ) {
    //noinspection ObjectEquality
    if ( selectedTag != tag ) {
      throw new IllegalArgumentException( "Tag not found: " + tag );
    }

    selectedTag = null;
    tcs.notifyTagRemoved( tag, 0 );
    return true;
  }

  @NotNull
  public List<? extends Tag> getTags() {
    if ( selectedTag != null ) {
      return Collections.singletonList( selectedTag );
    } else {
      return Collections.emptyList();
    }
  }

  public void addTagChangeListener( @NotNull TagChangeListener listener ) {
    tcs.addTagChangeListener( listener );
  }

  public void removeTagChangeListener( @NotNull TagChangeListener listener ) {
    tcs.removeTagChangeListener( listener );
  }

  @Nullable
  public Tag getSelectedTag() {
    return selectedTag;
  }
}
