package de.avat.tools.tags.ui;

import de.avat.tools.tags.SingleTaggable;
import de.avat.tools.tags.Tag;
import de.avat.tools.tags.TagManager;
import de.avat.tools.tags.TagObservable;
import de.avat.tools.tags.Taggable;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.ComboBoxModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

/**
 * ComboboxModel prsenting tags
 */
public class TagComboBoxModel extends TagListModel implements ComboBoxModel {
  private final SingleTaggable taggable = new SingleTaggable( this );

  public TagComboBoxModel( @NotNull TagObservable availableTags ) {
    this( availableTags, true );
  }

  public TagComboBoxModel( @NotNull TagObservable availableTags, boolean nullable ) {
    super( availableTags, nullable );
  }

  /**
   * Returns a taggable representing the selection
   *
   * @return the selection
   */
  @NotNull
  public Taggable getSelectionTaggable() {
    return taggable;
  }

  public void setSelectedItem( Object anItem ) {
    //noinspection ObjectEquality
    if ( anItem == getSelectedItem() ) {
      return;
    }

    final Tag newTag;

    if ( anItem == null || anItem instanceof Tag ) {
      newTag = ( Tag ) anItem;
    } else if ( anItem instanceof String ) {
      if ( ( ( CharSequence ) anItem ).length() == 0 ) {
        newTag = null;
      } else if ( availableTags instanceof Taggable ) {
        newTag = getTagManager().getTag( ( String ) anItem );
        ( ( Taggable ) availableTags ).addTag( newTag );
      } else {
        throw new IllegalStateException( "Cannot create a new tag on the fly. Need a Taggable" );
      }
    } else {
      throw new IllegalStateException( "???? " + anItem.getClass().getName() );
    }

    taggable.setSelectedTag( newTag );

    if ( !listeners.isEmpty() ) {
      ListDataEvent event = new ListDataEvent( this, ListDataEvent.CONTENTS_CHANGED, -1, -1 );
      for ( ListDataListener listener : listeners ) {
        listener.contentsChanged( event );
      }
    }
  }

  @Nullable
  public Tag getSelectedItem() {
    return taggable.getSelectedTag();
  }

  private TagManager<?> tagManager;

  @NotNull
  private TagManager<?> getTagManager() {
    if ( tagManager == null ) {
      if ( availableTags instanceof TagManager ) {
        return ( TagManager<?> ) availableTags;
      }
      throw new IllegalStateException( "No TagManager available" );
    }
    return tagManager;
  }

  public void setTagManager( @NotNull TagManager<?> tagManager ) {
    this.tagManager = tagManager;
  }
}
